@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -36,6 +42,7 @@
 #include "qv4objectproto_p.h"
 #include <private/qqmlvaluetypewrapper_p.h>
 #include <private/qv8engine_p.h>
+#include <private/qv4qobjectwrapper_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -43,20 +50,23 @@ using namespace QV4;
 
 DEFINE_OBJECT_VTABLE(VariantObject);
 
-Heap::VariantObject::VariantObject()
+void Heap::VariantObject::init()
 {
+    Object::init();
+    scarceData = new ExecutionEngine::ScarceResourceData;
 }
 
-Heap::VariantObject::VariantObject(const QVariant &value)
+void Heap::VariantObject::init(const QVariant &value)
 {
-    data = value;
+    Object::init();
+    scarceData = new ExecutionEngine::ScarceResourceData(value);
     if (isScarce())
-        internalClass->engine->scarceResources.insert(this);
+        removeVmePropertyReference();
 }
 
 bool VariantObject::Data::isScarce() const
 {
-    QVariant::Type t = data.type();
+    QVariant::Type t = data().type();
     return t == QVariant::Pixmap || t == QVariant::Image;
 }
 
@@ -66,10 +76,10 @@ bool VariantObject::isEqualTo(Managed *m, Managed *other)
     QV4::VariantObject *lv = static_cast<QV4::VariantObject *>(m);
 
     if (QV4::VariantObject *rv = other->as<QV4::VariantObject>())
-        return lv->d()->data == rv->d()->data;
+        return lv->d()->data() == rv->d()->data();
 
     if (QV4::QQmlValueTypeWrapper *v = other->as<QQmlValueTypeWrapper>())
-        return v->isEqual(lv->d()->data);
+        return v->isEqual(lv->d()->data());
 
     return false;
 }
@@ -80,7 +90,7 @@ void VariantObject::addVmePropertyReference()
         // remove from the ep->scarceResources list
         // since it is now no longer eligible to be
         // released automatically by the engine.
-        d()->node.remove();
+        d()->addVmePropertyReference();
     }
 }
 
@@ -90,7 +100,7 @@ void VariantObject::removeVmePropertyReference()
         // and add to the ep->scarceResources list
         // since it is now eligible to be released
         // automatically by the engine.
-        internalClass()->engine->scarceResources.insert(d());
+        d()->removeVmePropertyReference();
     }
 }
 
@@ -108,7 +118,7 @@ QV4::ReturnedValue VariantPrototype::method_preserve(CallContext *ctx)
     Scope scope(ctx);
     Scoped<VariantObject> o(scope, ctx->thisObject().as<QV4::VariantObject>());
     if (o && o->d()->isScarce())
-        o->d()->node.remove();
+        o->d()->addVmePropertyReference();
     return Encode::undefined();
 }
 
@@ -118,8 +128,8 @@ QV4::ReturnedValue VariantPrototype::method_destroy(CallContext *ctx)
     Scoped<VariantObject> o(scope, ctx->thisObject().as<QV4::VariantObject>());
     if (o) {
         if (o->d()->isScarce())
-            o->d()->node.remove();
-        o->d()->data = QVariant();
+            o->d()->addVmePropertyReference();
+        o->d()->data() = QVariant();
     }
     return Encode::undefined();
 }
@@ -130,9 +140,9 @@ QV4::ReturnedValue VariantPrototype::method_toString(CallContext *ctx)
     Scoped<VariantObject> o(scope, ctx->thisObject().as<QV4::VariantObject>());
     if (!o)
         return Encode::undefined();
-    QString result = o->d()->data.toString();
-    if (result.isEmpty() && !o->d()->data.canConvert(QVariant::String))
-        result = QStringLiteral("QVariant(%0)").arg(QString::fromLatin1(o->d()->data.typeName()));
+    QString result = o->d()->data().toString();
+    if (result.isEmpty() && !o->d()->data().canConvert(QVariant::String))
+        result = QStringLiteral("QVariant(%0)").arg(QString::fromLatin1(o->d()->data().typeName()));
     return Encode(ctx->d()->engine->newString(result));
 }
 
@@ -141,7 +151,7 @@ QV4::ReturnedValue VariantPrototype::method_valueOf(CallContext *ctx)
     Scope scope(ctx);
     Scoped<VariantObject> o(scope, ctx->thisObject().as<QV4::VariantObject>());
     if (o) {
-        QVariant v = o->d()->data;
+        QVariant v = o->d()->data();
         switch (v.type()) {
         case QVariant::Invalid:
             return Encode::undefined();
