@@ -41,7 +41,6 @@
 #include "qqmlcomponent_p.h"
 #include "qqmlcomponentattached_p.h"
 
-#include "qqmlcompiler_p.h"
 #include "qqmlcontext_p.h"
 #include "qqmlengine_p.h"
 #include "qqmlvme_p.h"
@@ -336,14 +335,11 @@ void QQmlComponentPrivate::typeDataProgress(QQmlTypeData *, qreal p)
 void QQmlComponentPrivate::fromTypeData(QQmlTypeData *data)
 {
     url = data->finalUrl();
-    QQmlCompiledData *c = data->compiledData();
+    compilationUnit = data->compilationUnit();
 
-    if (!c) {
+    if (!compilationUnit) {
         Q_ASSERT(data->isError());
         state.errors = data->errors();
-    } else {
-        cc = c;
-        cc->addref();
     }
 
     data->release();
@@ -357,10 +353,7 @@ void QQmlComponentPrivate::clear()
         typeData = 0;
     }
 
-    if (cc) {
-        cc->release();
-        cc = 0;
-    }
+    compilationUnit = nullptr;
 }
 
 /*!
@@ -394,8 +387,6 @@ QQmlComponent::~QQmlComponent()
         d->typeData->unregisterCallback(d);
         d->typeData->release();
     }
-    if (d->cc)
-        d->cc->release();
 }
 
 /*!
@@ -423,7 +414,7 @@ QQmlComponent::Status QQmlComponent::status() const
         return Loading;
     else if (!d->state.errors.isEmpty())
         return Error;
-    else if (d->engine && d->cc)
+    else if (d->engine && d->compilationUnit)
         return Ready;
     else
         return Null;
@@ -513,11 +504,8 @@ QQmlComponent::QQmlComponent(QQmlEngine *engine, QObject *parent)
     \sa loadUrl()
 */
 QQmlComponent::QQmlComponent(QQmlEngine *engine, const QUrl &url, QObject *parent)
-: QObject(*(new QQmlComponentPrivate), parent)
+    : QQmlComponent(engine, url, QQmlComponent::PreferSynchronous, parent)
 {
-    Q_D(QQmlComponent);
-    d->engine = engine;
-    d->loadUrl(url);
 }
 
 /*!
@@ -532,10 +520,9 @@ QQmlComponent::QQmlComponent(QQmlEngine *engine, const QUrl &url, QObject *paren
 */
 QQmlComponent::QQmlComponent(QQmlEngine *engine, const QUrl &url, CompilationMode mode,
                              QObject *parent)
-: QObject(*(new QQmlComponentPrivate), parent)
+    : QQmlComponent(engine, parent)
 {
     Q_D(QQmlComponent);
-    d->engine = engine;
     d->loadUrl(url, mode);
 }
 
@@ -547,12 +534,8 @@ QQmlComponent::QQmlComponent(QQmlEngine *engine, const QUrl &url, CompilationMod
 */
 QQmlComponent::QQmlComponent(QQmlEngine *engine, const QString &fileName,
                              QObject *parent)
-: QObject(*(new QQmlComponentPrivate), parent)
+    : QQmlComponent(engine, fileName, QQmlComponent::PreferSynchronous, parent)
 {
-    Q_D(QQmlComponent);
-    d->engine = engine;
-    const QUrl url = QDir::isAbsolutePath(fileName) ? QUrl::fromLocalFile(fileName) : d->engine->baseUrl().resolved(QUrl(fileName));
-    d->loadUrl(url);
 }
 
 /*!
@@ -564,10 +547,9 @@ QQmlComponent::QQmlComponent(QQmlEngine *engine, const QString &fileName,
 */
 QQmlComponent::QQmlComponent(QQmlEngine *engine, const QString &fileName,
                              CompilationMode mode, QObject *parent)
-: QObject(*(new QQmlComponentPrivate), parent)
+    : QQmlComponent(engine, parent)
 {
     Q_D(QQmlComponent);
-    d->engine = engine;
     const QUrl url = QDir::isAbsolutePath(fileName) ? QUrl::fromLocalFile(fileName) : d->engine->baseUrl().resolved(QUrl(fileName));
     d->loadUrl(url, mode);
 }
@@ -575,15 +557,13 @@ QQmlComponent::QQmlComponent(QQmlEngine *engine, const QString &fileName,
 /*!
     \internal
 */
-QQmlComponent::QQmlComponent(QQmlEngine *engine, QQmlCompiledData *cc, int start, QObject *parent)
-    : QObject(*(new QQmlComponentPrivate), parent)
+QQmlComponent::QQmlComponent(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *compilationUnit, int start, QObject *parent)
+    : QQmlComponent(engine, parent)
 {
     Q_D(QQmlComponent);
-    d->engine = engine;
-    d->cc = cc;
-    cc->addref();
+    d->compilationUnit = compilationUnit;
     d->start = start;
-    d->url = cc->url();
+    d->url = compilationUnit->url();
     d->progress = 1.0;
 }
 
@@ -876,7 +856,7 @@ QQmlComponentPrivate::beginCreate(QQmlContextData *context)
 
     enginePriv->referenceScarceResources();
     QObject *rv = 0;
-    state.creator.reset(new QQmlObjectCreator(context, cc, creationContext));
+    state.creator.reset(new QQmlObjectCreator(context, compilationUnit, creationContext));
     rv = state.creator->create(start);
     if (!rv)
         state.errors = state.creator->errors;
@@ -896,11 +876,13 @@ QQmlComponentPrivate::beginCreate(QQmlContextData *context)
         depthIncreased = false;
     }
 
-    QQmlEngineDebugService *service = QQmlDebugConnector::service<QQmlEngineDebugService>();
-    if (service && rv) {
-        if (!context->isInternal)
-            context->asQQmlContextPrivate()->instances.append(rv);
-        service->objectCreated(engine, rv);
+    if (rv) {
+        if (QQmlEngineDebugService *service =
+                QQmlDebugConnector::service<QQmlEngineDebugService>()) {
+            if (!context->isInternal)
+                context->asQQmlContextPrivate()->instances.append(rv);
+            service->objectCreated(engine, rv);
+        }
     }
 
     return rv;
@@ -917,7 +899,7 @@ void QQmlComponentPrivate::beginDeferred(QQmlEnginePrivate *enginePriv,
     Q_ASSERT(ddata->deferredData);
     QQmlData::DeferredData *deferredData = ddata->deferredData;
     QQmlContextData *creationContext = 0;
-    state->creator.reset(new QQmlObjectCreator(deferredData->context->parent, deferredData->compiledData, creationContext));
+    state->creator.reset(new QQmlObjectCreator(deferredData->context->parent, deferredData->compilationUnit, creationContext));
     if (!state->creator->populateDeferredProperties(object))
         state->errors << state->creator->errors;
 }
@@ -1061,9 +1043,9 @@ void QQmlComponent::create(QQmlIncubator &incubator, QQmlContext *context,
 
     QQmlEnginePrivate *enginePriv = QQmlEnginePrivate::get(d->engine);
 
-    p->compiledData = d->cc;
-    p->compiledData->addref();
-    p->creator.reset(new QQmlObjectCreator(contextData, d->cc, d->creationContext, p.data()));
+    p->compilationUnit = d->compilationUnit;
+    p->enginePriv = enginePriv;
+    p->creator.reset(new QQmlObjectCreator(contextData, d->compilationUnit, d->creationContext, p.data()));
     p->subComponentToCreate = d->start;
 
     enginePriv->incubate(incubator, forContextData);
@@ -1076,9 +1058,10 @@ namespace QV4 {
 namespace Heap {
 
 struct QmlIncubatorObject : Object {
-    QmlIncubatorObject(QQmlIncubator::IncubationMode = QQmlIncubator::Asynchronous);
-    QScopedPointer<QQmlComponentIncubator> incubator;
-    QPointer<QObject> parent;
+    void init(QQmlIncubator::IncubationMode = QQmlIncubator::Asynchronous);
+    inline void destroy();
+    QQmlComponentIncubator *incubator;
+    QQmlQPointer<QObject> parent;
     QV4::Value valuemap;
     QV4::Value statusChanged;
     Pointer<Heap::QmlContext> qmlContext;
@@ -1196,7 +1179,7 @@ static void QQmlComponent_setQmlParent(QObject *me, QObject *parent)
 */
 
 
-static void setInitialProperties(QV4::ExecutionEngine *engine, QV4::QmlContext *qmlContext, const QV4::Value &o, const QV4::Value &v)
+void QQmlComponentPrivate::setInitialProperties(QV4::ExecutionEngine *engine, QV4::QmlContext *qmlContext, const QV4::Value &o, const QV4::Value &v)
 {
     QV4::Scope scope(engine);
     QV4::ScopedObject object(scope);
@@ -1285,7 +1268,7 @@ void QQmlComponent::createObject(QQmlV4Function *args)
 
     if (!valuemap->isUndefined()) {
         QV4::Scoped<QV4::QmlContext> qmlContext(scope, v4->qmlContext());
-        setInitialProperties(v4, qmlContext, object, valuemap);
+        QQmlComponentPrivate::setInitialProperties(v4, qmlContext, object, valuemap);
     }
 
     d->completeCreate();
@@ -1406,7 +1389,7 @@ void QQmlComponent::incubateObject(QQmlV4Function *args)
     r->d()->qmlContext = v4->qmlContext();
     r->d()->parent = parent;
 
-    QQmlIncubator *incubator = r->d()->incubator.data();
+    QQmlIncubator *incubator = r->d()->incubator;
     create(*incubator, creationContext());
 
     if (incubator->status() == QQmlIncubator::Null) {
@@ -1501,12 +1484,20 @@ QQmlComponentExtension::~QQmlComponentExtension()
 {
 }
 
-QV4::Heap::QmlIncubatorObject::QmlIncubatorObject(QQmlIncubator::IncubationMode m)
-    : valuemap(QV4::Primitive::undefinedValue())
-    , statusChanged(QV4::Primitive::undefinedValue())
-    , qmlContext(0)
+void QV4::Heap::QmlIncubatorObject::init(QQmlIncubator::IncubationMode m)
 {
-    incubator.reset(new QQmlComponentIncubator(this, m));
+    Object::init();
+    valuemap = QV4::Primitive::undefinedValue();
+    statusChanged = QV4::Primitive::undefinedValue();
+    parent.init();
+    qmlContext = nullptr;
+    incubator = new QQmlComponentIncubator(this, m);
+}
+
+void QV4::Heap::QmlIncubatorObject::destroy() {
+    delete incubator;
+    parent.destroy();
+    Object::destroy();
 }
 
 void QV4::QmlIncubatorObject::setInitialState(QObject *o)
@@ -1518,7 +1509,7 @@ void QV4::QmlIncubatorObject::setInitialState(QObject *o)
         QV4::Scope scope(v4);
         QV4::ScopedObject obj(scope, QV4::QObjectWrapper::wrap(v4, o));
         QV4::Scoped<QV4::QmlContext> qmlCtxt(scope, d()->qmlContext);
-        setInitialProperties(v4, qmlCtxt, obj, d()->valuemap);
+        QQmlComponentPrivate::setInitialProperties(v4, qmlCtxt, obj, d()->valuemap);
     }
 }
 
@@ -1549,7 +1540,7 @@ void QV4::QmlIncubatorObject::statusChanged(QQmlIncubator::Status s)
         QV4::ScopedCallData callData(scope, 1);
         callData->thisObject = this;
         callData->args[0] = QV4::Primitive::fromUInt32(s);
-        f->call(callData);
+        f->call(scope, callData);
         if (scope.hasException()) {
             QQmlError error = scope.engine->catchExceptionAsQmlError();
             QQmlEnginePrivate::warning(QQmlEnginePrivate::get(scope.engine->qmlEngine()), error);
