@@ -191,7 +191,7 @@ void qmlClearEnginePlugins()
 {
     StringRegisteredPluginMap *plugins = qmlEnginePluginsWithRegisteredTypes();
     QMutexLocker lock(&plugins->mutex);
-    foreach (RegisteredPlugin plugin, plugins->values()) {
+    for (auto &plugin : qAsConst(*plugins)) {
         QPluginLoader* loader = plugin.loader;
         if (loader && !loader->unload())
             qWarning("Unloading %s failed: %s", qPrintable(plugin.uri), qPrintable(loader->errorString()));
@@ -399,9 +399,7 @@ bool excludeBaseUrl(const QString &importUrl, const QString &fileName, const QSt
 
     if (baseUrl.startsWith(importUrl))
     {
-        QString typeUrl(importUrl);
-        typeUrl.append(fileName);
-        if (typeUrl == baseUrl)
+        if (fileName == baseUrl.midRef(importUrl.size()))
             return false;
     }
 
@@ -540,10 +538,10 @@ QString QQmlImports::versionString(int vmaj, int vmin, ImportVersion version)
 {
     if (version == QQmlImports::FullyVersioned) {
         // extension with fully encoded version number (eg. MyModule.3.2)
-        return QString(QLatin1String(".%1.%2")).arg(vmaj).arg(vmin);
+        return QString::asprintf(".%d.%d", vmaj, vmin);
     } else if (version == QQmlImports::PartiallyVersioned) {
         // extension with encoded version major (eg. MyModule.3)
-        return QString(QLatin1String(".%1")).arg(vmaj);
+        return QString::asprintf(".%d", vmaj);
     } // else extension without version number (eg. MyModule)
     return QString();
 }
@@ -913,7 +911,7 @@ bool QQmlImportsPrivate::populatePluginPairVector(QVector<StaticPluginPair> &res
         // To avoid traversing all static plugins for all imports, we cut down
         // the list the first time called to only contain QML plugins:
         foreach (const QStaticPlugin &plugin, QPluginLoader::staticPlugins()) {
-            if (qobject_cast<QQmlExtensionPlugin *>(plugin.instance()))
+            if (plugin.metaData().value(QLatin1String("IID")).toString() == QLatin1String(QQmlExtensionInterface_iid))
                 plugins.append(plugin);
         }
     }
@@ -921,7 +919,7 @@ bool QQmlImportsPrivate::populatePluginPairVector(QVector<StaticPluginPair> &res
     foreach (const QStaticPlugin &plugin, plugins) {
         // Since a module can list more than one plugin, we keep iterating even after we found a match.
         if (QQmlExtensionPlugin *instance = qobject_cast<QQmlExtensionPlugin *>(plugin.instance())) {
-            const QJsonArray metaTagsUriList = plugin.metaData().value(QStringLiteral("uri")).toArray();
+            const QJsonArray metaTagsUriList = plugin.metaData().value(QLatin1String("uri")).toArray();
             if (metaTagsUriList.isEmpty()) {
                 if (errors) {
                     QQmlError error;
@@ -1395,15 +1393,9 @@ bool QQmlImportsPrivate::addFileImport(const QString& uri, const QString &prefix
     // The uri for this import.  For library imports this is the same as uri
     // specified by the user, but it may be different in the case of file imports.
     QString importUri = uri;
-
-    QString qmldirPath = importUri;
-    if (importUri.endsWith(Slash))
-        qmldirPath += String_qmldir;
-    else
-        qmldirPath += Slash_qmldir;
-
-    QString qmldirUrl = resolveLocalUrl(base, qmldirPath);
-
+    QString qmldirUrl = resolveLocalUrl(base, importUri + (importUri.endsWith(Slash)
+                                                           ? String_qmldir
+                                                           : Slash_qmldir));
     QString qmldirIdentifier;
 
     if (QQmlFile::isLocalFile(qmldirUrl)) {
@@ -1701,13 +1693,9 @@ QString QQmlImportDatabase::resolvePlugin(QQmlTypeLoader *typeLoader,
         if (!resolvedPath.endsWith(Slash))
             resolvedPath += Slash;
 
+        resolvedPath += prefix + baseName;
         foreach (const QString &suffix, suffixes) {
-            QString pluginFileName = prefix;
-
-            pluginFileName += baseName;
-            pluginFileName += suffix;
-
-            QString absolutePath = typeLoader->absoluteFilePath(resolvedPath + pluginFileName);
+            const QString absolutePath = typeLoader->absoluteFilePath(resolvedPath + suffix);
             if (!absolutePath.isEmpty())
                 return absolutePath;
         }
@@ -1739,29 +1727,32 @@ QString QQmlImportDatabase::resolvePlugin(QQmlTypeLoader *typeLoader,
                                                   const QString &baseName)
 {
 #if defined(Q_OS_WIN)
-    return resolvePlugin(typeLoader, qmldirPath, qmldirPluginPath, baseName,
-                         QStringList()
+    static const QString prefix;
+    static const QStringList suffixes = {
 # ifdef QT_DEBUG
-                         << QLatin1String("d.dll") // try a qmake-style debug build first
+        QLatin1String("d.dll"), // try a qmake-style debug build first
 # endif
-                         << QLatin1String(".dll"));
+        QLatin1String(".dll")
+    };
 #elif defined(Q_OS_DARWIN)
-
-    return resolvePlugin(typeLoader, qmldirPath, qmldirPluginPath, baseName,
-                         QStringList()
+    static const QString prefix = QLatin1String("lib");
+    static const QStringList suffixes = {
 # ifdef QT_DEBUG
-                         << QLatin1String("_debug.dylib") // try a qmake-style debug build first
-                         << QLatin1String(".dylib")
+        QLatin1String("_debug.dylib"), // try a qmake-style debug build first
+        QLatin1String(".dylib"),
 # else
-                         << QLatin1String(".dylib")
-                         << QLatin1String("_debug.dylib") // try a qmake-style debug build after
+        QLatin1String(".dylib"),
+        QLatin1String("_debug.dylib"), // try a qmake-style debug build after
 # endif
-                         << QLatin1String(".so")
-                         << QLatin1String(".bundle"),
-                         QLatin1String("lib"));
+        QLatin1String(".so"),
+        QLatin1String(".bundle")
+    };
 # else  // Unix
-    return resolvePlugin(typeLoader, qmldirPath, qmldirPluginPath, baseName, QStringList() << QLatin1String(".so"), QLatin1String("lib"));
+    static const QString prefix = QLatin1String("lib");
+    static const QStringList suffixes = { QLatin1String(".so") };
 #endif
+
+    return resolvePlugin(typeLoader, qmldirPath, qmldirPluginPath, baseName, suffixes, prefix);
 }
 
 /*!
@@ -1820,7 +1811,7 @@ void QQmlImportDatabase::addImportPath(const QString& path)
     } else if (path.startsWith(QLatin1Char(':'))) {
         // qrc directory, e.g. :/foo
         // need to convert to a qrc url, e.g. qrc:/foo
-        cPath = QStringLiteral("qrc") + path;
+        cPath = QLatin1String("qrc") + path;
         cPath.replace(Backslash, Slash);
     } else if (url.isRelative() ||
                (url.scheme().length() == 1 && QFile::exists(path)) ) {  // windows path
@@ -1960,7 +1951,7 @@ bool QQmlImportDatabase::importStaticPlugin(QObject *instance, const QString &ba
 #ifndef QT_NO_LIBRARY
     // Dynamic plugins are differentiated by their filepath. For static plugins we
     // don't have that information so we use their address as key instead.
-    QString uniquePluginID = QString().sprintf("%p", instance);
+    const QString uniquePluginID = QString::asprintf("%p", instance);
     StringRegisteredPluginMap *plugins = qmlEnginePluginsWithRegisteredTypes();
     QMutexLocker lock(&plugins->mutex);
 
