@@ -55,6 +55,8 @@
 #include <QtCore/qmath.h>
 #include "qplatformdefs.h"
 
+#include <cmath>
+
 QT_BEGIN_NAMESPACE
 
 // FlickThreshold determines how far the "mouse" must have moved
@@ -69,6 +71,21 @@ static const int RetainGrabVelocity = 100;
 static const int MovementEndingTimerInterval = 100;
 #endif
 
+// Currently std::round can't be used on Android when using ndk g++, so
+// use C version instead. We could just define two versions of Round, one
+// for float and one for double, but then only one of them would be used
+// and compiler would trigger a warning about unused function.
+//
+// See https://code.google.com/p/android/issues/detail?id=54418
+template<typename T>
+static T Round(T t) {
+    return round(t);
+}
+template<>
+Q_DECL_UNUSED float Round<float>(float f) {
+    return roundf(f);
+}
+
 static qreal EaseOvershoot(qreal t) {
     return qAtan(t);
 }
@@ -292,14 +309,14 @@ void QQuickFlickablePrivate::AxisData::updateVelocity()
     }
 }
 
-void QQuickFlickablePrivate::itemGeometryChanged(QQuickItem *item, const QRectF &newGeom, const QRectF &oldGeom)
+void QQuickFlickablePrivate::itemGeometryChanged(QQuickItem *item, QQuickGeometryChange change, const QRectF &)
 {
     Q_Q(QQuickFlickable);
     if (item == contentItem) {
         Qt::Orientations orient = 0;
-        if (newGeom.x() != oldGeom.x())
+        if (change.xChange())
             orient |= Qt::Horizontal;
-        if (newGeom.y() != oldGeom.y())
+        if (change.yChange())
             orient |= Qt::Vertical;
         if (orient)
             q->viewportMoved(orient);
@@ -351,7 +368,7 @@ bool QQuickFlickablePrivate::flick(AxisData &data, qreal minExtent, qreal maxExt
         qreal dist = v2 / (accel * 2.0);
         if (v > 0)
             dist = -dist;
-        qreal target = -qRound(-(data.move.value() - dist));
+        qreal target = -Round(-(data.move.value() - dist));
         dist = -target + data.move.value();
         accel = v2 / (2.0f * qAbs(dist));
 
@@ -455,18 +472,18 @@ void QQuickFlickablePrivate::fixup(AxisData &data, qreal minExtent, qreal maxExt
     } else if (data.move.value() <= maxExtent) {
         resetTimeline(data);
         adjustContentPos(data, maxExtent);
-    } else if (-qRound(-data.move.value()) != data.move.value()) {
+    } else if (-Round(-data.move.value()) != data.move.value()) {
         // We could animate, but since it is less than 0.5 pixel it's probably not worthwhile.
         resetTimeline(data);
         qreal val = data.move.value();
-        if (qAbs(-qRound(-val) - val) < 0.25) // round small differences
-            val = -qRound(-val);
+        if (std::abs(-Round(-val) - val) < 0.25) // round small differences
+            val = -Round(-val);
         else if (data.smoothVelocity.value() > 0) // continue direction of motion for larger
-            val = -qFloor(-val);
+            val = -std::floor(-val);
         else if (data.smoothVelocity.value() < 0)
-            val = -qCeil(-val);
+            val = -std::ceil(-val);
         else // otherwise round
-            val = -qRound(-val);
+            val = -Round(-val);
         timeline.set(data.move, val);
     }
     data.inOvershoot = false;
@@ -1553,12 +1570,12 @@ void QQuickFlickablePrivate::replayDelayedPress()
 //XXX pixelAligned ignores the global position of the Flickable, i.e. assumes Flickable itself is pixel aligned.
 void QQuickFlickablePrivate::setViewportX(qreal x)
 {
-    contentItem->setX(pixelAligned ? -qRound(-x) : x);
+    contentItem->setX(pixelAligned ? -Round(-x) : x);
 }
 
 void QQuickFlickablePrivate::setViewportY(qreal y)
 {
-    contentItem->setY(pixelAligned ? -qRound(-y) : y);
+    contentItem->setY(pixelAligned ? -Round(-y) : y);
 }
 
 void QQuickFlickable::timerEvent(QTimerEvent *event)
@@ -2219,21 +2236,28 @@ void QQuickFlickablePrivate::cancelInteraction()
     }
 }
 
-bool QQuickFlickable::sendMouseEvent(QQuickItem *item, QMouseEvent *event)
+/*!
+    QQuickFlickable::filterMouseEvent checks filtered mouse events and potentially steals them.
+
+    This is how flickable takes over events from other items (\a receiver) that are on top of it.
+    It filters their events and may take over (grab) the \a event.
+    Return true if the mouse event will be stolen.
+    \internal
+*/
+bool QQuickFlickable::filterMouseEvent(QQuickItem *receiver, QMouseEvent *event)
 {
     Q_D(QQuickFlickable);
     QPointF localPos = mapFromScene(event->windowPos());
 
-    QQuickWindow *c = window();
-    QQuickItem *grabber = c ? c->mouseGrabberItem() : 0;
-    if (grabber == this && d->stealMouse) {
+    Q_ASSERT_X(receiver != this, "", "Flickable received a filter event for itself");
+    if (receiver == this && d->stealMouse) {
         // we are already the grabber and we do want the mouse event to ourselves.
         return true;
     }
 
-    bool grabberDisabled = grabber && !grabber->isEnabled();
+    bool receiverDisabled = receiver && !receiver->isEnabled();
     bool stealThisEvent = d->stealMouse;
-    if ((stealThisEvent || contains(localPos)) && (!grabber || !grabber->keepMouseGrab() || grabberDisabled)) {
+    if ((stealThisEvent || contains(localPos)) && (!receiver || !receiver->keepMouseGrab() || receiverDisabled)) {
         QScopedPointer<QMouseEvent> mouseEvent(QQuickWindowPrivate::cloneMouseEvent(event, &localPos));
         mouseEvent->setAccepted(false);
 
@@ -2243,7 +2267,7 @@ bool QQuickFlickable::sendMouseEvent(QQuickItem *item, QMouseEvent *event)
             break;
         case QEvent::MouseButtonPress:
             d->handleMousePressEvent(mouseEvent.data());
-            d->captureDelayedPress(item, event);
+            d->captureDelayedPress(receiver, event);
             stealThisEvent = d->stealMouse;   // Update stealThisEvent in case changed by function call above
             break;
         case QEvent::MouseButtonRelease:
@@ -2253,15 +2277,14 @@ bool QQuickFlickable::sendMouseEvent(QQuickItem *item, QMouseEvent *event)
         default:
             break;
         }
-        grabber = qobject_cast<QQuickItem*>(c->mouseGrabberItem());
-        if ((grabber && stealThisEvent && !grabber->keepMouseGrab() && grabber != this) || grabberDisabled) {
+        if ((receiver && stealThisEvent && !receiver->keepMouseGrab() && receiver != this) || receiverDisabled) {
             d->clearDelayedPress();
             grabMouse();
         } else if (d->delayedPressEvent) {
             grabMouse();
         }
 
-        const bool filtered = stealThisEvent || d->delayedPressEvent || grabberDisabled;
+        const bool filtered = stealThisEvent || d->delayedPressEvent || receiverDisabled;
         if (filtered) {
             event->setAccepted(true);
         }
@@ -2270,7 +2293,7 @@ bool QQuickFlickable::sendMouseEvent(QQuickItem *item, QMouseEvent *event)
         d->lastPosTime = -1;
         returnToBounds();
     }
-    if (event->type() == QEvent::MouseButtonRelease || (grabber && grabber->keepMouseGrab() && !grabberDisabled)) {
+    if (event->type() == QEvent::MouseButtonRelease || (receiver && receiver->keepMouseGrab() && !receiverDisabled)) {
         // mouse released, or another item has claimed the grab
         d->lastPosTime = -1;
         d->clearDelayedPress();
@@ -2290,7 +2313,7 @@ bool QQuickFlickable::childMouseEventFilter(QQuickItem *i, QEvent *e)
     case QEvent::MouseButtonPress:
     case QEvent::MouseMove:
     case QEvent::MouseButtonRelease:
-        return sendMouseEvent(i, static_cast<QMouseEvent *>(e));
+        return filterMouseEvent(i, static_cast<QMouseEvent *>(e));
     case QEvent::UngrabMouse:
         if (d->window && d->window->mouseGrabberItem() && d->window->mouseGrabberItem() != this) {
             // The grab has been taken away from a child and given to some other item.
