@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -67,11 +65,12 @@ QQuickLoaderPrivate::~QQuickLoaderPrivate()
     disposeInitialPropertyValues();
 }
 
-void QQuickLoaderPrivate::itemGeometryChanged(QQuickItem *resizeItem, const QRectF &newGeometry, const QRectF &oldGeometry)
+void QQuickLoaderPrivate::itemGeometryChanged(QQuickItem *resizeItem, QQuickGeometryChange change,
+                                              const QRectF &oldGeometry)
 {
     if (resizeItem == item)
         _q_updateSize(false);
-    QQuickItemChangeListener::itemGeometryChanged(resizeItem, newGeometry, oldGeometry);
+    QQuickItemChangeListener::itemGeometryChanged(resizeItem, change, oldGeometry);
 }
 
 void QQuickLoaderPrivate::itemImplicitWidthChanged(QQuickItem *)
@@ -225,7 +224,7 @@ qreal QQuickLoaderPrivate::getImplicitHeight() const
     loads \c MyItem.qml, and is able to receive the \c message signal from
     the loaded item through a \l Connections object:
 
-    \table
+    \table 70%
     \row
     \li application.qml
     \li MyItem.qml
@@ -271,7 +270,6 @@ qreal QQuickLoaderPrivate::getImplicitHeight() const
 
     In some cases you may wish to use a Loader within a view delegate to improve delegate
     loading performance. This works well in most cases, but there is one important issue to
-    be aware of related to the \l{QtQml::Component#Creation Context}{creation context} of a Component.
 
     In the following example, the \c index context property inserted by the ListView into \c delegateComponent's
     context will be inaccessible to Text, as the Loader will use the creation context of \c myComponent as the parent
@@ -474,7 +472,7 @@ void QQuickLoader::setSourceComponent(QQmlComponent *comp)
     d->component = comp;
     if (comp) {
         if (QQmlData *ddata = QQmlData::get(comp))
-            d->componentStrongReference = ddata->jsWrapper.value();
+            d->componentStrongReference = ddata->jsWrapper;
     }
     d->loadingFromSource = false;
 
@@ -524,7 +522,7 @@ void QQuickLoader::loadFromSourceComponent()
     is changed after calling this function but prior to setting the loader \l active.
 
     Example:
-    \table
+    \table 70%
     \row
     \li
     \qml
@@ -553,11 +551,13 @@ void QQuickLoader::loadFromSourceComponent()
     Item {
         Loader {
             id: squareLoader
-            onLoaded: console.log(squareLoader.item.width); // prints [10], not [30]
+            onLoaded: console.log(squareLoader.item.width);
+            // prints [10], not [30]
         }
 
         Component.onCompleted: {
-            squareLoader.setSource("ExampleComponent.qml", { "color": "blue" });
+            squareLoader.setSource("ExampleComponent.qml",
+                                 { "color": "blue" });
             // will trigger the onLoaded code when complete.
         }
     }
@@ -582,9 +582,9 @@ void QQuickLoader::setSource(QQmlV4Function *args)
     QUrl sourceUrl = d->resolveSourceUrl(args);
     if (!ipv->isUndefined()) {
         d->disposeInitialPropertyValues();
-        d->initialPropertyValues = ipv.asReturnedValue();
-        d->qmlGlobalForIpv = args->qmlGlobal();
+        d->initialPropertyValues.set(args->v4engine(), ipv);
     }
+    d->qmlCallingContext.set(scope.engine, scope.engine->qmlContext());
 
     setSource(sourceUrl, false); // already cleared and set ipv above.
 }
@@ -649,11 +649,12 @@ void QQuickLoaderPrivate::setInitialState(QObject *obj)
 
     QQmlComponentPrivate *d = QQmlComponentPrivate::get(component);
     Q_ASSERT(d && d->engine);
-    QV4::ExecutionEngine *v4 = qmlGlobalForIpv.engine();
+    QV4::ExecutionEngine *v4 = QV8Engine::getV4(d->engine);
     Q_ASSERT(v4);
     QV4::Scope scope(v4);
     QV4::ScopedValue ipv(scope, initialPropertyValues.value());
-    d->initializeObjectWithInitialProperties(qmlGlobalForIpv, ipv, obj);
+    QV4::Scoped<QV4::QmlContext> qmlContext(scope, qmlCallingContext.value());
+    d->initializeObjectWithInitialProperties(qmlContext, ipv, obj);
 }
 
 void QQuickLoaderIncubator::statusChanged(Status status)
@@ -860,6 +861,11 @@ likelihood of glitches in animation.  When loading asynchronously the status
 will change to Loader.Loading.  Once the entire component has been created, the
 \l item will be available and the status will change to Loader.Ready.
 
+Changing the value of this property to \c false while an asynchronous load is in
+progress will force immediate, synchronous completion.  This allows beginning an
+asynchronous load and then forcing completion if the Loader content must be
+accessed before the asynchronous load has completed.
+
 To avoid seeing the items loading progressively set \c visible appropriately, e.g.
 
 \code
@@ -886,6 +892,19 @@ void QQuickLoader::setAsynchronous(bool a)
         return;
 
     d->asynchronous = a;
+
+    if (!d->asynchronous && isComponentComplete() && d->active) {
+        if (d->loadingFromSource && d->component && d->component->isLoading()) {
+            // Force a synchronous component load
+            QUrl currentSource = d->source;
+            d->clear();
+            d->source = currentSource;
+            loadFromSource();
+        } else if (d->incubator && d->incubator->isLoading()) {
+            d->incubator->forceCompletion();
+        }
+    }
+
     emit asynchronousChanged();
 }
 
@@ -939,7 +958,7 @@ QUrl QQuickLoaderPrivate::resolveSourceUrl(QQmlV4Function *args)
     if (arg.isEmpty())
         return QUrl();
 
-    QQmlContextData *context = args->context();
+    QQmlContextData *context = scope.engine->callingQmlContext();
     Q_ASSERT(context);
     return context->resolvedUrl(QUrl(arg));
 }
@@ -950,7 +969,7 @@ QV4::ReturnedValue QQuickLoaderPrivate::extractInitialPropertyValues(QQmlV4Funct
     QV4::ScopedValue valuemap(scope, QV4::Primitive::undefinedValue());
     if (args->length() >= 2) {
         QV4::ScopedValue v(scope, (*args)[1]);
-        if (!v->isObject() || v->asArrayObject()) {
+        if (!v->isObject() || v->as<QV4::ArrayObject>()) {
             *error = true;
             qmlInfo(loader) << QQuickLoader::tr("setSource: value is not an object");
         } else {
@@ -959,7 +978,7 @@ QV4::ReturnedValue QQuickLoaderPrivate::extractInitialPropertyValues(QQmlV4Funct
         }
     }
 
-    return valuemap.asReturnedValue();
+    return valuemap->asReturnedValue();
 }
 
 #include <moc_qquickloader_p.cpp>
