@@ -99,10 +99,12 @@ QSGGlyphNode *QQuickTextNode::addGlyphs(const QPointF &position, const QGlyphRun
     bool preferNativeGlyphNode = m_useNativeRenderer;
     if (!preferNativeGlyphNode) {
         QRawFontPrivate *fontPriv = QRawFontPrivate::get(font);
-        if (fontPriv->fontEngine->hasUnreliableGlyphOutline())
+        if (fontPriv->fontEngine->hasUnreliableGlyphOutline()) {
             preferNativeGlyphNode = true;
-        else
-            preferNativeGlyphNode = !QFontDatabase().isSmoothlyScalable(font.familyName(), font.styleName());
+        } else {
+            QFontEngine *fe = QRawFontPrivate::get(font)->fontEngine;
+            preferNativeGlyphNode = !fe->isSmoothlyScalable;
+        }
     }
 
     QSGGlyphNode *node = sg->sceneGraphContext()->createGlyphNode(sg, preferNativeGlyphNode);
@@ -136,7 +138,7 @@ void QQuickTextNode::setCursor(const QRectF &rect, const QColor &color)
         delete m_cursorNode;
 
     QSGRenderContext *sg = QQuickItemPrivate::get(m_ownerElement)->sceneGraphRenderContext();
-    m_cursorNode =  sg->sceneGraphContext()->createRectangleNode(rect, color);
+    m_cursorNode =  sg->sceneGraphContext()->createInternalRectangleNode(rect, color);
     appendChildNode(m_cursorNode);
 }
 
@@ -151,27 +153,23 @@ void QQuickTextNode::clearCursor()
 void QQuickTextNode::addRectangleNode(const QRectF &rect, const QColor &color)
 {
     QSGRenderContext *sg = QQuickItemPrivate::get(m_ownerElement)->sceneGraphRenderContext();
-    appendChildNode(sg->sceneGraphContext()->createRectangleNode(rect, color));
+    appendChildNode(sg->sceneGraphContext()->createInternalRectangleNode(rect, color));
 }
 
 
 void QQuickTextNode::addImage(const QRectF &rect, const QImage &image)
 {
     QSGRenderContext *sg = QQuickItemPrivate::get(m_ownerElement)->sceneGraphRenderContext();
-    QSGImageNode *node = sg->sceneGraphContext()->createImageNode();
+    QSGInternalImageNode *node = sg->sceneGraphContext()->createInternalImageNode();
     QSGTexture *texture = sg->createTexture(image);
-    if (m_ownerElement->smooth()) {
+    if (m_ownerElement->smooth())
         texture->setFiltering(QSGTexture::Linear);
-        texture->setMipmapFiltering(QSGTexture::Linear);
-    }
     m_textures.append(texture);
     node->setTargetRect(rect);
     node->setInnerTargetRect(rect);
     node->setTexture(texture);
-    if (m_ownerElement->smooth()) {
+    if (m_ownerElement->smooth())
         node->setFiltering(QSGTexture::Linear);
-        node->setMipmapFiltering(QSGTexture::Linear);
-    }
     appendChildNode(node);
     node->update();
 }
