@@ -778,6 +778,18 @@ QSGBasicGeometryNode::~QSGBasicGeometryNode()
     \internal
  */
 
+/*!
+    \fn void QSGBasicGeometryNode::setRendererMatrix(const QMatrix4x4 *m)
+
+    \internal
+ */
+
+/*!
+    \fn void QSGBasicGeometryNode::setRendererClipList(const QSGClipNode *c)
+
+    \internal
+ */
+
 
 /*!
     Sets the geometry of this node to \a geometry.
@@ -1256,7 +1268,7 @@ QSGRootNode::QSGRootNode()
 QSGRootNode::~QSGRootNode()
 {
     while (!m_renderers.isEmpty())
-        m_renderers.last()->setRootNode(0);
+        m_renderers.constLast()->setRootNode(0);
     destroy(); // Must call destroy() here because markDirty() casts this to QSGRootNode.
 }
 
@@ -1454,8 +1466,6 @@ void QSGNodeVisitor::visitChildren(QSGNode *n)
         visitNode(c);
 }
 
-
-
 #ifndef QT_NO_DEBUG_STREAM
 QDebug operator<<(QDebug d, const QSGGeometryNode *n)
 {
@@ -1472,15 +1482,15 @@ QDebug operator<<(QDebug d, const QSGGeometryNode *n)
     } else {
 
         switch (g->drawingMode()) {
-        case GL_TRIANGLE_STRIP: d << "strip"; break;
-        case GL_TRIANGLE_FAN: d << "fan"; break;
-        case GL_TRIANGLES: d << "triangles"; break;
+        case QSGGeometry::DrawTriangleStrip: d << "strip"; break;
+        case QSGGeometry::DrawTriangleFan: d << "fan"; break;
+        case QSGGeometry::DrawTriangles: d << "triangles"; break;
         default: break;
         }
 
         d << "#V:" << g->vertexCount() << "#I:" << g->indexCount();
 
-        if (g->attributeCount() > 0 && g->attributes()->type == GL_FLOAT) {
+        if (g->attributeCount() > 0 && g->attributes()->type == QSGGeometry::FloatType) {
             float x1 = 1e10, x2 = -1e10, y1=1e10, y2=-1e10;
             int stride = g->sizeOfVertex();
             for (int i = 0; i < g->vertexCount(); ++i) {
