@@ -38,7 +38,6 @@
 ****************************************************************************/
 
 #include "qquickpixmapcache_p.h"
-#include <qqmlnetworkaccessmanagerfactory.h>
 #include <qquickimageprovider.h>
 
 #include <qqmlengine.h>
@@ -55,7 +54,6 @@
 #include <QCoreApplication>
 #include <QImageReader>
 #include <QHash>
-#include <QNetworkReply>
 #include <QPixmapCache>
 #include <QFile>
 #include <QThread>
@@ -66,10 +64,15 @@
 #include <QWaitCondition>
 #include <QtCore/qdebug.h>
 #include <private/qobject_p.h>
-#include <QSslError>
 #include <QQmlFile>
 #include <QMetaMethod>
 
+#if QT_CONFIG(qml_network)
+#include <qqmlnetworkaccessmanagerfactory.h>
+#include <QNetworkReply>
+#include <QSslError>
+#endif
+
 #include <private/qquickprofiler_p.h>
 
 #define IMAGEREQUEST_MAX_NETWORK_REQUEST_COUNT 8
@@ -203,7 +206,9 @@ private:
     friend class QQuickPixmapReaderThreadObject;
     void processJobs();
     void processJob(QQuickPixmapReply *, const QUrl &, const QString &, AutoTransform, QQuickImageProvider::ImageType, QQuickImageProvider *);
+#if QT_CONFIG(qml_network)
     void networkRequestDone(QNetworkReply *);
+#endif
     void asyncResponseFinished(QQuickImageResponse *);
 
     QList<QQuickPixmapReply*> jobs;
@@ -215,10 +220,11 @@ private:
     QQuickPixmapReaderThreadObject *threadObject;
     QWaitCondition waitCondition;
 
+#if QT_CONFIG(qml_network)
     QNetworkAccessManager *networkAccessManager();
     QNetworkAccessManager *accessManager;
-
     QHash<QNetworkReply*,QQuickPixmapReply*> networkJobs;
+#endif
     QHash<QQuickImageResponse*,QQuickPixmapReply*> asyncResponses;
 
     static int replyDownloadProgress;
@@ -343,6 +349,7 @@ QQuickPixmapReply::Event::~Event()
     delete textureFactory;
 }
 
+#if QT_CONFIG(qml_network)
 QNetworkAccessManager *QQuickPixmapReader::networkAccessManager()
 {
     if (!accessManager) {
@@ -351,6 +358,7 @@ QNetworkAccessManager *QQuickPixmapReader::networkAccessManager()
     }
     return accessManager;
 }
+#endif
 
 static void maybeRemoveAlpha(QImage *image)
 {
@@ -421,7 +429,10 @@ static bool readImage(const QUrl& url, QIODevice *dev, QImage *image, QString *e
 }
 
 QQuickPixmapReader::QQuickPixmapReader(QQmlEngine *eng)
-: QThread(eng), engine(eng), threadObject(0), accessManager(0)
+: QThread(eng), engine(eng), threadObject(0)
+#if QT_CONFIG(qml_network)
+, accessManager(0)
+#endif
 {
     eventLoopQuitHack = new QObject;
     eventLoopQuitHack->moveToThread(this);
@@ -437,19 +448,27 @@ QQuickPixmapReader::~QQuickPixmapReader()
 
     mutex.lock();
     // manually cancel all outstanding jobs.
-    foreach (QQuickPixmapReply *reply, jobs) {
+    for (QQuickPixmapReply *reply : qAsConst(jobs)) {
         if (reply->data && reply->data->reply == reply)
             reply->data->reply = 0;
         delete reply;
     }
     jobs.clear();
-    QList<QQuickPixmapReply*> activeJobs = networkJobs.values() + asyncResponses.values();
-    foreach (QQuickPixmapReply *reply, activeJobs ) {
+#if QT_CONFIG(qml_network)
+
+    const auto cancelJob = [this](QQuickPixmapReply *reply) {
         if (reply->loading) {
             cancelled.append(reply);
             reply->data = 0;
         }
-    }
+    };
+
+    for (auto *reply : qAsConst(networkJobs))
+        cancelJob(reply);
+
+    for (auto *reply : qAsConst(asyncResponses))
+        cancelJob(reply);
+#endif
     if (threadObject) threadObject->processJobs();
     mutex.unlock();
 
@@ -457,6 +476,7 @@ QQuickPixmapReader::~QQuickPixmapReader()
     wait();
 }
 
+#if QT_CONFIG(qml_network)
 void QQuickPixmapReader::networkRequestDone(QNetworkReply *reply)
 {
     QQuickPixmapReply *job = networkJobs.take(reply);
@@ -506,6 +526,7 @@ void QQuickPixmapReader::networkRequestDone(QNetworkReply *reply)
     // kick off event loop again incase we have dropped below max request count
     threadObject->processJobs();
 }
+#endif // qml_network
 
 void QQuickPixmapReader::asyncResponseFinished(QQuickImageResponse *response)
 {
@@ -556,8 +577,10 @@ bool QQuickPixmapReaderThreadObject::event(QEvent *e)
 
 void QQuickPixmapReaderThreadObject::networkRequestDone()
 {
+#if QT_CONFIG(qml_network)
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
     reader->networkRequestDone(reply);
+#endif
 }
 
 void QQuickPixmapReaderThreadObject::asyncResponseFinished()
@@ -576,6 +599,7 @@ void QQuickPixmapReader::processJobs()
 
         // Clean cancelled jobs
         if (!cancelled.isEmpty()) {
+#if QT_CONFIG(qml_network)
             for (int i = 0; i < cancelled.count(); ++i) {
                 QQuickPixmapReply *job = cancelled.at(i);
                 QNetworkReply *reply = networkJobs.key(job, 0);
@@ -597,13 +621,14 @@ void QQuickPixmapReader::processJobs()
                 job->deleteLater();
             }
             cancelled.clear();
+#endif
         }
 
         if (!jobs.isEmpty()) {
             // Find a job we can use
             bool usableJob = false;
             for (int i = jobs.count() - 1; !usableJob && i >= 0; i--) {
-                QQuickPixmapReply *job = jobs[i];
+                QQuickPixmapReply *job = jobs.at(i);
                 const QUrl url = job->url;
                 QString localFile;
                 QQuickImageProvider::ImageType imageType = QQuickImageProvider::Invalid;
@@ -617,7 +642,11 @@ void QQuickPixmapReader::processJobs()
                     usableJob = true;
                 } else {
                     localFile = QQmlFile::urlToLocalFileOrQrc(url);
-                    usableJob = !localFile.isEmpty() || networkJobs.count() < IMAGEREQUEST_MAX_NETWORK_REQUEST_COUNT;
+                    usableJob = !localFile.isEmpty()
+#if QT_CONFIG(qml_network)
+                            || networkJobs.count() < IMAGEREQUEST_MAX_NETWORK_REQUEST_COUNT
+#endif
+                            ;
                 }
 
 
@@ -742,6 +771,7 @@ void QQuickPixmapReader::processJob(QQuickPixmapReply *runningJob, const QUrl &u
                 runningJob->postReply(errorCode, errorStr, readSize, QQuickTextureFactory::textureFactoryForImage(image));
             mutex.unlock();
         } else {
+#if QT_CONFIG(qml_network)
             // Network resource
             QNetworkRequest req(url);
             req.setAttribute(QNetworkRequest::HttpPipeliningAllowedAttribute, true);
@@ -751,6 +781,9 @@ void QQuickPixmapReader::processJob(QQuickPixmapReply *runningJob, const QUrl &u
             QMetaObject::connect(reply, replyFinished, threadObject, threadNetworkRequestDone);
 
             networkJobs.insert(reply, runningJob);
+#else
+// Silently fail if compiled with no_network
+#endif
         }
     }
 }
@@ -807,11 +840,13 @@ void QQuickPixmapReader::cancel(QQuickPixmapReply *reply)
 void QQuickPixmapReader::run()
 {
     if (replyDownloadProgress == -1) {
+#if QT_CONFIG(qml_network)
         replyDownloadProgress = QMetaMethod::fromSignal(&QNetworkReply::downloadProgress).methodIndex();
         replyFinished = QMetaMethod::fromSignal(&QNetworkReply::finished).methodIndex();
-        downloadProgress = QMetaMethod::fromSignal(&QQuickPixmapReply::downloadProgress).methodIndex();
         const QMetaObject *ir = &QQuickPixmapReaderThreadObject::staticMetaObject;
         threadNetworkRequestDone = ir->indexOfSlot("networkRequestDone()");
+#endif
+        downloadProgress = QMetaMethod::fromSignal(&QQuickPixmapReply::downloadProgress).methodIndex();
     }
 
     mutex.lock();
@@ -886,15 +921,13 @@ QQuickPixmapStore::~QQuickPixmapStore()
 #ifndef QT_NO_DEBUG
     int leakedPixmaps = 0;
 #endif
-    QList<QQuickPixmapData*> cachedData = m_cache.values();
-
     // Prevent unreferencePixmap() from assuming it needs to kick
     // off the cache expiry timer, as we're shrinking the cache
     // manually below after releasing all the pixmaps.
     m_timerId = -2;
 
     // unreference all (leaked) pixmaps
-    foreach (QQuickPixmapData* pixmap, cachedData) {
+    for (auto *pixmap : qAsConst(m_cache)) {
         int currRefCount = pixmap->refCount;
         if (currRefCount) {
 #ifndef QT_NO_DEBUG
