@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -45,6 +43,7 @@
 #include "qquickanimation_p_p.h"
 #include "private/qcontinuinganimationgroupjob_p.h"
 
+#include <qmath.h>
 #include <qqmlproperty.h>
 #include <private/qqmlproperty_p.h>
 
@@ -52,7 +51,6 @@
 
 #include <QtCore/qdebug.h>
 
-#include <math.h>
 
 #define DELAY_STOP_TIMER_INTERVAL 32
 
@@ -165,7 +163,7 @@ bool QSmoothedAnimation::recalc()
         return false;
     }
 
-    finalDuration = ceil(tf * 1000.0);
+    finalDuration = qCeil(tf * 1000.0);
 
     if (maximumEasingTime == 0) {
         a = 0;
@@ -199,7 +197,7 @@ bool QSmoothedAnimation::recalc()
         qreal c2 = 0.5 * vi * tf - s;
         qreal c3 = -0.25 * vi * vi;
 
-        qreal a1 = (-c2 + sqrt(c2 * c2 - 4 * c1 * c3)) / (2. * c1);
+        qreal a1 = (-c2 + qSqrt(c2 * c2 - 4 * c1 * c3)) / (2. * c1);
 
         qreal tp1 = 0.5 * tf - 0.5 * vi / a1;
         qreal vp1 = a1 * tp1 + vi;
@@ -256,8 +254,8 @@ void QSmoothedAnimation::updateCurrentTime(int t)
     qreal value = easeFollow(time_seconds);
     value *= (invert? -1.0: 1.0);
     QQmlPropertyPrivate::write(target, initialValue + value,
-                                       QQmlPropertyPrivate::BypassInterceptor
-                                       | QQmlPropertyPrivate::DontRemoveBinding);
+                                       QQmlPropertyData::BypassInterceptor
+                                       | QQmlPropertyData::DontRemoveBinding);
 }
 
 void QSmoothedAnimation::init()
@@ -289,8 +287,8 @@ void QSmoothedAnimation::init()
                 break;
             case QQuickSmoothedAnimation::Sync:
                 QQmlPropertyPrivate::write(target, to,
-                                                   QQmlPropertyPrivate::BypassInterceptor
-                                                   | QQmlPropertyPrivate::DontRemoveBinding);
+                                                   QQmlPropertyData::BypassInterceptor
+                                                   | QQmlPropertyData::DontRemoveBinding);
                 trackVelocity = 0;
                 stop();
                 return;
@@ -306,8 +304,8 @@ void QSmoothedAnimation::init()
 
     if (!recalc()) {
         QQmlPropertyPrivate::write(target, to,
-                                           QQmlPropertyPrivate::BypassInterceptor
-                                           | QQmlPropertyPrivate::DontRemoveBinding);
+                                           QQmlPropertyData::BypassInterceptor
+                                           | QQmlPropertyData::DontRemoveBinding);
         stop();
         return;
     }
@@ -315,7 +313,7 @@ void QSmoothedAnimation::init()
 
 void QSmoothedAnimation::debugAnimation(QDebug d) const
 {
-    d << "SmoothedAnimationJob(" << hex << (void *) this << dec << ")" << "duration:" << userDuration
+    d << "SmoothedAnimationJob(" << hex << (const void *) this << dec << ")" << "duration:" << userDuration
       << "velocity:" << velocity << "target:" << target.object() << "property:" << target.name()
       << "to:" << to << "current velocity:" << trackVelocity;
 }
@@ -379,23 +377,22 @@ QQuickSmoothedAnimation::~QQuickSmoothedAnimation()
 }
 
 QQuickSmoothedAnimationPrivate::QQuickSmoothedAnimationPrivate()
-    : anim(0)
+    : anim(new QSmoothedAnimation)
 {
-    anim = new QSmoothedAnimation;
 }
 
 QQuickSmoothedAnimationPrivate::~QQuickSmoothedAnimationPrivate()
 {
+    typedef QHash<QQmlProperty, QSmoothedAnimation* >::iterator ActiveAnimationsHashIt;
+
     delete anim;
-    QHash<QQmlProperty, QSmoothedAnimation* >::iterator it;
-    for (it = activeAnimations.begin(); it != activeAnimations.end(); ++it) {
+    for (ActiveAnimationsHashIt it = activeAnimations.begin(), end = activeAnimations.end(); it != end; ++it)
         it.value()->clearTemplate();
-    }
 }
 
 void QQuickSmoothedAnimationPrivate::updateRunningAnimations()
 {
-    foreach(QSmoothedAnimation* ease, activeAnimations.values()){
+    for (QSmoothedAnimation *ease : qAsConst(activeAnimations)) {
         ease->maximumEasingTime = anim->maximumEasingTime;
         ease->reversingMode = anim->reversingMode;
         ease->velocity = anim->velocity;
@@ -412,7 +409,7 @@ QAbstractAnimationJob* QQuickSmoothedAnimation::transition(QQuickStateActions &a
     Q_UNUSED(direction);
     Q_D(QQuickSmoothedAnimation);
 
-    QQuickStateActions dataActions = QQuickPropertyAnimation::createTransitionActions(actions, modified, defaultTarget);
+    const QQuickStateActions dataActions = QQuickPropertyAnimation::createTransitionActions(actions, modified, defaultTarget);
 
     QContinuingAnimationGroupJob *wrapperGroup = new QContinuingAnimationGroupJob();
 
@@ -447,7 +444,8 @@ QAbstractAnimationJob* QQuickSmoothedAnimation::transition(QQuickStateActions &a
             anims.insert(ease);
         }
 
-        foreach (QSmoothedAnimation *ease, d->activeAnimations.values()){
+        const auto copy = d->activeAnimations;
+        for (QSmoothedAnimation *ease : copy) {
             if (!anims.contains(ease)) {
                 ease->clearTemplate();
                 d->activeAnimations.remove(ease->target);
