@@ -42,11 +42,6 @@
 #include <QtCore/qlibraryinfo.h>
 #include <QtQml/qjsengine.h>
 
-#if defined (Q_OS_WINCE)
-#undef IN
-#undef OUT
-#endif
-
 const char *V8REQUEST = "v8request";
 const char *V8MESSAGE = "v8message";
 const char *SEQ = "seq";
