@@ -43,7 +43,6 @@
 #include <QtQml/qqmlpropertyvaluesource.h>
 #include <QtQml/qqmlscriptstring.h>
 #include <QtQml/qqmlproperty.h>
-#include <private/qqmlcompiler_p.h>
 #include <private/qqmlcustomparser_p.h>
 
 QVariant myCustomVariantTypeConverter(const QString &data);
@@ -731,9 +730,19 @@ private:
 
 
 namespace MyNamespace {
+    Q_NAMESPACE
+    enum MyNSEnum {
+        Key1 = 1,
+        Key2,
+        Key5 = 5
+    };
+    Q_ENUM_NS(MyNSEnum);
+
     class MyNamespacedType : public QObject
     {
         Q_OBJECT
+        Q_PROPERTY(MyNamespace::MyNSEnum myEnum MEMBER m_myEnum)
+        MyNamespace::MyNSEnum m_myEnum = MyNSEnum::Key1;
     };
 
     class MySecondNamespacedType : public QObject
@@ -757,14 +766,14 @@ class MyCustomParserTypeParser : public QQmlCustomParser
 {
 public:
     virtual void verifyBindings(const QV4::CompiledData::Unit *, const QList<const QV4::CompiledData::Binding *> &) {}
-    virtual void applyBindings(QObject *, QQmlCompiledData *, const QList<const QV4::CompiledData::Binding *> &) {}
+    virtual void applyBindings(QObject *, QV4::CompiledData::CompilationUnit *, const QList<const QV4::CompiledData::Binding *> &) {}
 };
 
 class EnumSupportingCustomParser : public QQmlCustomParser
 {
 public:
     virtual void verifyBindings(const QV4::CompiledData::Unit *, const QList<const QV4::CompiledData::Binding *> &);
-    virtual void applyBindings(QObject *, QQmlCompiledData *, const QList<const QV4::CompiledData::Binding *> &) {}
+    virtual void applyBindings(QObject *, QV4::CompiledData::CompilationUnit *, const QList<const QV4::CompiledData::Binding *> &) {}
 };
 
 class MyParserStatus : public QObject, public QQmlParserStatus
@@ -1114,6 +1123,58 @@ public:
     static QObject *qmlAttachedProperties(QObject *parent) { return new QObject(parent); }
 };
 
+class MyArrayBufferTestClass : public QObject
+{
+    Q_OBJECT
+    Q_PROPERTY(QByteArray byteArrayProperty READ byteArrayProperty WRITE setByteArrayProperty NOTIFY byteArrayPropertyChanged)
+
+signals:
+    void byteArrayPropertyChanged();
+    void byteArraySignal(QByteArray arg);
+
+public:
+    QByteArray byteArrayPropertyValue;
+    QByteArray byteArrayProperty() const {
+        return byteArrayPropertyValue;
+    }
+    void setByteArrayProperty(const QByteArray &v) {
+        byteArrayPropertyValue = v;
+        emit byteArrayPropertyChanged();
+    }
+    Q_INVOKABLE void emitByteArraySignal(char begin, char num) {
+        byteArraySignal(byteArrayMethod_CountUp(begin, num));
+    }
+    Q_INVOKABLE int byteArrayMethod_Sum(QByteArray arg) {
+        int sum = 0;
+        for (int i = 0; i < arg.size(); ++i) {
+            sum += arg[i];
+        }
+        return sum;
+    }
+    Q_INVOKABLE QByteArray byteArrayMethod_CountUp(char begin, int num) {
+        QByteArray ret;
+        for (int i = 0; i < num; ++i) {
+            ret.push_back(begin++);
+        }
+        return ret;
+    }
+    Q_INVOKABLE bool byteArrayMethod_Overloaded(QByteArray) {
+        return true;
+    }
+    Q_INVOKABLE bool byteArrayMethod_Overloaded(int) {
+        return false;
+    }
+    Q_INVOKABLE bool byteArrayMethod_Overloaded(QString) {
+        return false;
+    }
+    Q_INVOKABLE bool byteArrayMethod_Overloaded(QJSValue) {
+        return false;
+    }
+    Q_INVOKABLE bool byteArrayMethod_Overloaded(QVariant) {
+        return false;
+    }
+};
+
 Q_DECLARE_METATYPE(MyEnum2Class::EnumB)
 Q_DECLARE_METATYPE(MyEnum1Class::EnumA)
 Q_DECLARE_METATYPE(Qt::TextFormat)
@@ -1142,7 +1203,7 @@ public:
     void setTarget(QObject *newTarget) { m_target = newTarget; }
 
     QPointer<QObject> m_target;
-    QQmlRefPointer<QQmlCompiledData> cdata;
+    QQmlRefPointer<QV4::CompiledData::CompilationUnit> compilationUnit;
     QList<const QV4::CompiledData::Binding*> bindings;
     QByteArray m_bindingData;
 };
@@ -1150,7 +1211,7 @@ public:
 class CustomBindingParser : public QQmlCustomParser
 {
     virtual void verifyBindings(const QV4::CompiledData::Unit *, const QList<const QV4::CompiledData::Binding *> &) {}
-    virtual void applyBindings(QObject *, QQmlCompiledData *, const QList<const QV4::CompiledData::Binding *> &);
+    virtual void applyBindings(QObject *, QV4::CompiledData::CompilationUnit *, const QList<const QV4::CompiledData::Binding *> &);
 };
 
 class SimpleObjectWithCustomParser : public QObject
@@ -1196,7 +1257,7 @@ private:
 class SimpleObjectCustomParser : public QQmlCustomParser
 {
     virtual void verifyBindings(const QV4::CompiledData::Unit *, const QList<const QV4::CompiledData::Binding *> &) {}
-    virtual void applyBindings(QObject *, QQmlCompiledData *, const QList<const QV4::CompiledData::Binding *> &);
+    virtual void applyBindings(QObject *, QV4::CompiledData::CompilationUnit *, const QList<const QV4::CompiledData::Binding *> &);
 };
 
 class RootObjectInCreationTester : public QObject
