@@ -93,6 +93,8 @@ private slots:
     void cleanup();
     void contentSize();
     void ratios_smallContent();
+    void contentXYNotTruncatedToInt();
+    void keepGrab();
 
 private:
     void flickWithTouch(QQuickWindow *window, QTouchDevice *touchDevice, const QPoint &from, const QPoint &to);
@@ -1223,7 +1225,9 @@ void tst_qquickflickable::flickOnRelease()
     QTRY_VERIFY(!flickable->isMoving());
 
 #ifdef Q_OS_MAC
+# ifndef QT_NO_OPENGL
     QEXPECT_FAIL("", "QTBUG-26094 stopping on a full pixel doesn't work on OS X", Continue);
+# endif
 #endif
     // Stop on a full pixel after user interaction
     QCOMPARE(flickable->contentY(), (qreal)qRound(flickable->contentY()));
@@ -1966,6 +1970,72 @@ void tst_qquickflickable::ratios_smallContent()
     QCOMPARE(obj->property("widthRatioIs").toDouble(), 1.);
 }
 
+// QTBUG-48018
+void tst_qquickflickable::contentXYNotTruncatedToInt()
+{
+    QScopedPointer<QQuickView> window(new QQuickView);
+    window->setSource(testFileUrl("contentXY.qml"));
+    QTRY_COMPARE(window->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(window.data());
+    QQuickViewTestUtil::moveMouseAway(window.data());
+    window->show();
+    QVERIFY(QTest::qWaitForWindowActive(window.data()));
+
+    QQuickFlickable *flickable = qobject_cast<QQuickFlickable*>(window->rootObject());
+    QVERIFY(flickable);
+
+    flickable->setContentX(1e10);
+    flick(window.data(), QPoint(200, 100), QPoint(100, 100), 50);
+
+    // make sure we are not clipped at 2^31
+    QVERIFY(flickable->contentX() > qreal(1e10));
+}
+
+void tst_qquickflickable::keepGrab()
+{
+    QScopedPointer<QQuickView> window(new QQuickView);
+    window->setSource(testFileUrl("keepGrab.qml"));
+    QTRY_COMPARE(window->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(window.data());
+    QQuickViewTestUtil::moveMouseAway(window.data());
+    window->show();
+    QVERIFY(QTest::qWaitForWindowActive(window.data()));
+
+    QQuickFlickable *flickable = qobject_cast<QQuickFlickable*>(window->rootObject());
+    QVERIFY(flickable);
+
+    QQuickMouseArea *ma = flickable->findChild<QQuickMouseArea*>("ma");
+    QVERIFY(ma);
+    ma->setPreventStealing(true);
+
+    QPoint pos(250, 250);
+    moveAndPress(window.data(), pos);
+    for (int i = 0; i < 6; ++i) {
+        pos += QPoint(10, 10);
+        QTest::mouseMove(window.data(), pos);
+        QTest::qWait(10);
+    }
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(310, 310));
+    QTest::qWait(10);
+
+    QCOMPARE(flickable->contentX(), 0.0);
+    QCOMPARE(flickable->contentY(), 0.0);
+
+    ma->setPreventStealing(false);
+
+    pos = QPoint(250, 250);
+    moveAndPress(window.data(), pos);
+    for (int i = 0; i < 6; ++i) {
+        pos += QPoint(10, 10);
+        QTest::mouseMove(window.data(), pos);
+        QTest::qWait(10);
+    }
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(310, 310));
+    QTest::qWait(10);
+
+    QVERIFY(flickable->contentX() != 0.0);
+    QVERIFY(flickable->contentY() != 0.0);
+}
 
 QTEST_MAIN(tst_qquickflickable)
 
