@@ -175,10 +175,10 @@ QFileInfoList findQmlFiles(const QString &dirName)
 
     QFileInfoList ret;
     if (dir.exists()) {
-        QFileInfoList fileInfos = dir.entryInfoList(QStringList() << "*.qml",
-                                                    QDir::Files | QDir::AllDirs | QDir::NoDotAndDotDot);
+        const QFileInfoList fileInfos = dir.entryInfoList(QStringList() << "*.qml",
+                                                          QDir::Files | QDir::AllDirs | QDir::NoDotAndDotDot);
 
-        foreach (QFileInfo fileInfo, fileInfos) {
+        for (const QFileInfo &fileInfo : fileInfos) {
             if (fileInfo.isDir())
                 ret += findQmlFiles(fileInfo.filePath());
             else if (fileInfo.fileName().length() > 0 && fileInfo.fileName().at(0).isLower())
@@ -196,9 +196,9 @@ static int displayOptionsDialog(Options *options)
     QFormLayout *layout = new QFormLayout(&dialog);
 
     QComboBox *qmlFileComboBox = new QComboBox(&dialog);
-    QFileInfoList fileInfos = findQmlFiles(":/bundle") + findQmlFiles("./qmlscene-resources");
+    const QFileInfoList fileInfos = findQmlFiles(":/bundle") + findQmlFiles("./qmlscene-resources");
 
-    foreach (QFileInfo fileInfo, fileInfos)
+    for (const QFileInfo &fileInfo : fileInfos)
         qmlFileComboBox->addItem(fileInfo.dir().dirName() + QLatin1Char('/') + fileInfo.fileName(), QVariant::fromValue(fileInfo));
 
     QCheckBox *originalCheckBox = new QCheckBox(&dialog);
@@ -319,8 +319,8 @@ static void loadDummyDataFiles(QQmlEngine &engine, const QString& directory)
         QObject *dummyData = comp.create();
 
         if(comp.isError()) {
-            QList<QQmlError> errors = comp.errors();
-            foreach (const QQmlError &error, errors)
+            const QList<QQmlError> errors = comp.errors();
+            for (const QQmlError &error : errors)
                 fprintf(stderr, "%s\n", qPrintable(error.toString()));
         }
 
@@ -361,7 +361,7 @@ static void usage()
     puts(" ");
     exit(1);
 }
-
+#ifndef QT_NO_OPENGL
 // Listen on GL context creation of the QQuickWindow in order to print diagnostic output.
 class DiagnosticGlContextCreationListener : public QObject {
     Q_OBJECT
@@ -389,7 +389,9 @@ private slots:
         context->doneCurrent();
         deleteLater();
     }
+
 };
+#endif
 
 static void setWindowTitle(bool verbose, const QObject *topLevel, QWindow *window)
 {
@@ -403,8 +405,10 @@ static void setWindowTitle(bool verbose, const QObject *topLevel, QWindow *windo
     if (verbose) {
         newTitle += QLatin1String(" [Qt ") + QLatin1String(QT_VERSION_STR) + QLatin1Char(' ')
             + QGuiApplication::platformName() + QLatin1Char(' ');
+#ifndef QT_NO_OPENGL
         newTitle += QOpenGLContext::openGLModuleType() == QOpenGLContext::LibGL
             ? QLatin1String("GL") : QLatin1String("GLES");
+#endif
         newTitle += QLatin1Char(']');
     }
     if (oldTitle != newTitle)
@@ -453,7 +457,7 @@ int main(int argc, char ** argv)
             options.applicationAttributes.append(Qt::AA_DisableHighDpiScaling);
     }
 
-    foreach (Qt::ApplicationAttribute a, options.applicationAttributes)
+    for (Qt::ApplicationAttribute a : qAsConst(options.applicationAttributes))
         QCoreApplication::setAttribute(a);
 #ifdef QT_WIDGETS_LIB
     QApplication app(argc, argv);
@@ -463,6 +467,7 @@ int main(int argc, char ** argv)
     app.setApplicationName("QtQmlViewer");
     app.setOrganizationName("QtProject");
     app.setOrganizationDomain("qt-project.org");
+    QCoreApplication::setApplicationVersion(QLatin1String(QT_VERSION_STR));
 
     const QStringList arguments = QCoreApplication::arguments();
     for (int i = 1, size = arguments.size(); i < size; ++i) {
@@ -560,6 +565,7 @@ int main(int argc, char ** argv)
                 loadDummyDataFiles(engine, fi.path());
             }
             QObject::connect(&engine, SIGNAL(quit()), QCoreApplication::instance(), SLOT(quit()));
+            QObject::connect(&engine, &QQmlEngine::exit, QCoreApplication::instance(), &QCoreApplication::exit);
             component->loadUrl(options.url);
             while (component->isLoading())
                 QCoreApplication::processEvents();
@@ -592,8 +598,10 @@ int main(int argc, char ** argv)
 
             if (window) {
                 setWindowTitle(options.verbose, topLevel, window.data());
+#ifndef QT_NO_OPENGL
                 if (options.verbose)
                     new DiagnosticGlContextCreationListener(window.data());
+#endif
                 QSurfaceFormat surfaceFormat = window->requestedFormat();
                 if (options.multisample)
                     surfaceFormat.setSamples(16);
